﻿Imports System.Windows.Forms
Imports System.Drawing

Module Module_SaveLoad

    Friend EventsAreEnabled As Boolean = False
    Friend RecogTextFile As String = "Colors_ENG"
    Friend SpeechTextFile As String = "Test1_ENG"

    ' =======================================================================================================
    '   APP TITLE AND VERSION
    ' =======================================================================================================
    Friend Function AppTitleAndVersion(Optional ByVal Title As String = "") As String
        If Title = "" Then Title = Replace(My.Application.Info.AssemblyName, "_", " ")
        Dim s() As String = Split(My.Application.Info.Version.ToString, ".")
        Return Title & " - V" & s(0) & "." & s(1)
    End Function


    ' =======================================================================================
    '  FORM FUNCTIONS
    ' =======================================================================================
    Friend Sub LimitFormPosition(ByVal f As System.Windows.Forms.Form)
        If f.WindowState <> FormWindowState.Normal Then Return
        GetMaxScreenBounds()
        EnsureFormVisible(f)
        'EnsureFormCompletelyVisible(f)
    End Sub

    Private SB As Rectangle = New Rectangle(Integer.MaxValue, Integer.MaxValue, Integer.MinValue, Integer.MinValue)

    Private Sub GetMaxScreenBounds()
        For Each s As Screen In System.Windows.Forms.Screen.AllScreens
            SB = Rectangle.Union(SB, s.WorkingArea)
        Next
    End Sub

    'Private Sub EnsureFormCompletelyVisible(ByVal frm As Form)
    '    With frm
    '        .Width = Math.Min(.Width, SB.Width)         ' not more than a maximized window
    '        .Height = Math.Min(.Height, SB.Height)      ' not more than a maximized window
    '        .Width = Math.Max(.Width, 32)               ' at least 32x24
    '        .Height = Math.Max(.Height, 24)             ' at least 32x24
    '        .Left = Math.Min(.Left, SB.Right - .Width)  ' not beyond the right border
    '        .Top = Math.Min(.Top, SB.Bottom - .Height)  ' not beyond the bottom border
    '        .Left = Math.Max(.Left, SB.Left)            ' at least at the left border
    '        .Top = Math.Max(.Top, SB.Top)               ' at least at the top border
    '    End With
    'End Sub

    Private Sub EnsureFormVisible(ByVal frm As Form)
        With frm
            .Width = Math.Min(.Width, SB.Width)             ' not more than VIRTUALSCREEN dimensions
            .Height = Math.Min(.Height, SB.Height)          ' not more than VIRTUALSCREEN dimensions 
            .Width = Math.Max(.Width, 32)                   ' at least 32x24
            .Height = Math.Max(.Height, 24)                 ' at least 32x24
            .Left = Math.Min(.Left, SB.Right - 50)          ' not beyond right border - 50 pixels
            .Top = Math.Min(.Top, SB.Bottom - 100)          ' not beyond bottom border - 50 pixels
            .Left = Math.Max(.Left, SB.Left + 100 - .Width) ' at least at left border + 50 pixels
            .Top = Math.Max(.Top, SB.Top - 10)              ' at least at top border
        End With
    End Sub

    ' (The value of the RestoreBounds property is valid only 
    '   when the WindowState property of the Form class is not equal to Normal)
    Friend Function GetFormRectangle(ByVal frm As Form) As Rectangle
        Dim r As Rectangle
        If frm.WindowState = FormWindowState.Normal Then
            r = frm.Bounds
        Else
            r = frm.RestoreBounds
        End If
        Return r
    End Function


    ' ================================================================================================
    '  Private Read-Write functions
    ' ================================================================================================
    Private Function TabString(ByVal Name As String, _
                              Optional ByVal Value As Double = Double.NaN, _
                              Optional ByVal fmt As String = "") As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)
        If Double.IsNaN(Value) Then
            Return Name
        Else
            Return Name & "=" & Strings.StrDup(nTab, " ") & Value.ToString(fmt)
        End If
    End Function

    Private Function TabString(ByVal Name As String, _
                                  ByVal Value As Boolean) As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)

        Return Name & "=" & Strings.StrDup(nTab, " ") & Value.ToString
    End Function

    Private Function TabString(ByVal Name As String, _
                                ByVal Value As String) As String

        Dim nTab As Int32 = Math.Max(0, 22 - Name.Length)

        Return Name & "=" & Strings.StrDup(nTab, " ") & Value
    End Function

    Private Function Val_Single(ByVal l As String) As Single
        Return CSng(Val(l.Replace(",", ".")))
    End Function

    Private Function Val_Double(ByVal l As String) As Double
        Return Val(l.Replace(",", "."))
    End Function

    Private Function Val_Int(ByVal l As String) As Int32
        Return CInt(Val(l))
    End Function

    Private Function ExtractParamName(ByRef s As String) As String
        ' ------------------------- Returns the first field from begin to the first "=" symbol
        ' -------------------------  and removes it from the string
        Dim i As Integer
        i = InStr(s, "=")
        If i > 0 Then
            ExtractParamName = Trim(Strings.Left(s, i - 1))
            s = Trim(Mid(s, i + 1))
        Else
            ExtractParamName = Trim(s)
            s = ""
        End If
    End Function

    Private Function AssemblyName() As String
        Return System.Reflection.Assembly.GetExecutingAssembly.GetName.Name
    End Function


    ' ==================================================================================================
    '  SAVE LOAD -- Program INI
    ' ==================================================================================================
    Friend Sub Save_INI()
        '
        Dim iniFileName As String = Application.StartupPath & "\" & AssemblyName() & "_INI.txt"
        Dim f As System.IO.StreamWriter = Nothing
        Try
            f = IO.File.CreateText(iniFileName)
            '
            f.WriteLine(" Program Params")
            f.WriteLine("===========================================")
            ' ------------------------------------------------------------------------------ FORM BOUNDS
            Dim r As Rectangle
            r = GetFormRectangle(Form1)
            f.WriteLine(TabString("Form1_Top", r.Top))
            f.WriteLine(TabString("Form1_Left", r.Left))
            f.WriteLine(TabString("Form1_Width", r.Width))
            f.WriteLine(TabString("Form1_Height", r.Height))
            f.WriteLine(TabString("Form1_WindowState", Form1.WindowState))
            f.WriteLine(TabString("Form1_SplitterDistance1", Form1.SplitContainer1.SplitterDistance))
            f.WriteLine(TabString("Form1_SplitterDistance2", Form1.SplitContainer2.SplitterDistance))
            '
            f.WriteLine(TabString(""))
            f.WriteLine(TabString(" Slots"))
            f.WriteLine("===========================================")
            f.WriteLine(TabString("VoiceIndex", Form1.GetVoiceIndex))
            f.WriteLine(TabString("RecogTextFile", RecogTextFile))
            f.WriteLine(TabString("SpeechTextFile", SpeechTextFile))
            f.WriteLine(TabString("SayValidWords", Form1.SayType))
            f.WriteLine(TabString("SayValidTitle", Form1.SayValidPrefixText))
            f.WriteLine(TabString("Running", Form1.Running))
            f.WriteLine(TabString("StopRecognition", Form1.StopRecognition))
        Catch
        End Try
        Try
            f.Close()
        Catch
        End Try

        SaveRecogFile()
        SaveSpeechFile()
    End Sub

    Friend Sub Load_INI()
        ' ------------------------------------------------------------------------------- defaults
        '
        ' -------------------------------------------------------------------------------
        ' ------------------------------------------------------------------------------- 
        ' With "Resume Next" subsequent parameters are loaded and f.Close() is executed
        ' -------------------------------------------------------------------------------
        On Error Resume Next  ' use Resume-Next instead of Try-Catch
        ' -------------------------------------------------------------------------------
        Dim l As String
        Dim iniFileName As String = Application.StartupPath & "\" & AssemblyName() & "_INI.txt"
        If My.Computer.FileSystem.FileExists(iniFileName) Then

            Dim f As System.IO.StreamReader
            f = IO.File.OpenText(iniFileName)

            Do While Not f.EndOfStream
                l = f.ReadLine()
                Select Case ExtractParamName(l)
                    Case "Form1_Top" : Form1.Top = Val_Int(l)
                    Case "Form1_Left" : Form1.Left = Val_Int(l)
                    Case "Form1_Width" : Form1.Width = Val_Int(l)
                    Case "Form1_Height" : Form1.Height = Val_Int(l)
                    Case "Form1_WindowState" : Form1.WindowState = CType((Val(l)), FormWindowState)
                    Case "Form1_SplitterDistance1" : Form1.SplitContainer1.SplitterDistance = Val_Int(l)
                    Case "Form1_SplitterDistance2" : Form1.SplitContainer2.SplitterDistance = Val_Int(l)
                        ' ----------------------------------------------
                    Case "VoiceIndex" : Form1.SetVoiceIndex(Val_Int(l))
                    Case "RecogTextFile" : RecogTextFile = l
                    Case "SpeechTextFile" : SpeechTextFile = l
                    Case "SayValidWords" : Form1.SayType = CType(Val_Int(l), Form1.SayTypes)
                    Case "SayValidTitle" : Form1.SayValidPrefixText = l
                    Case "Running" : Form1.Running = l = "True"
                    Case "StopRecognition" : Form1.StopRecognition = l = "True"
                End Select
            Loop
            f.Close()
        End If
        LimitFormPosition(Form1)
        ' -----------------------------------------------
        ReadRecogFile()
        ReadSpeechFile()
    End Sub


    ' ==================================================================================================
    '  SAVE LOAD -- RECOG AND SPEECH FILES
    ' ==================================================================================================
    Private RecogPath As String = Application.StartupPath + "\Recog_Files\"
    Private SpeechPath As String = Application.StartupPath + "\Speech_Files\"
    Friend Sub ReadRecogFile()
        Dim fname As String = RecogPath + RecogTextFile + ".txt"
        If My.Computer.FileSystem.FileExists(fname) Then
            Form1.txt_ValidWords.Text = ReadTextFile(fname)
            Form1.txt_ValidWords.SelectionLength = 0
            Form1.lbl_Recog.Text = "Recognition   ( " + RecogTextFile + " )"
        End If
    End Sub
    Friend Sub ReadSpeechFile()
        Dim fname As String = SpeechPath + SpeechTextFile + ".txt"
        If My.Computer.FileSystem.FileExists(fname) Then
            Form1.txt_TTS.Text = ReadTextFile(fname)
            Form1.txt_TTS.SelectionLength = 0
            Form1.lbl_Speech.Text = "Speech   ( " + SpeechTextFile + " )"
        End If
    End Sub
    Friend Sub SaveRecogFile()
        If Form1.txt_ValidWords.Text = "" Then Return
        WriteTextFile(RecogPath + RecogTextFile + ".txt", Form1.txt_ValidWords.Text)
        Form1.lbl_Recog.Text = "Recognition   ( " + RecogTextFile + " )"
    End Sub
    Friend Sub SaveSpeechFile()
        If Form1.txt_TTS.Text = "" Then Return
        WriteTextFile(SpeechPath + SpeechTextFile + ".txt", Form1.txt_TTS.Text)
        Form1.lbl_Speech.Text = "Speech   ( " + SpeechTextFile + " )"
    End Sub

    Friend Sub WriteTextFile(ByVal fname As String, ByVal text As String)
        'text = Replace(text, vbLf, vbCrLf)
        Dim writer As System.IO.StreamWriter
        writer = New System.IO.StreamWriter(fname)
        writer.Write(text)
        writer.Close()
    End Sub

    Friend Function ReadTextFile(ByVal fname As String) As String
        Dim reader As System.IO.TextReader
        reader = New System.IO.StreamReader(fname)
        Dim text As String = reader.ReadToEnd
        reader.Close()
        'text = Replace(text, vbLf, vbCrLf)
        'text = Replace(text, vbCrLf, vbLf)
        'text = Replace(text, vbCr, vbLf)
        'text = Replace(text, vbLf, "")
        'text = Replace(text, vbCr, "")
        Return text
    End Function

    Friend Sub RecogSave()
        Dim sfd As SaveFileDialog = New SaveFileDialog()
        sfd.InitialDirectory = RecogPath
        sfd.DefaultExt = ".txt"
        sfd.AddExtension = True
        sfd.FileName = RecogTextFile
        sfd.Filter = "Recognition text file (*.txt)|*txt"
        If sfd.ShowDialog() = Windows.Forms.DialogResult.OK Then
            RecogTextFile = IO.Path.GetFileNameWithoutExtension(sfd.FileName)
            SaveRecogFile()
        End If
    End Sub

    Friend Sub RecogLoad()
        Dim ofd As OpenFileDialog = New OpenFileDialog()
        ofd.InitialDirectory = RecogPath
        ofd.Filter = "Recognition text file (*.txt)|*txt"
        ofd.DefaultExt = ".txt"
        ofd.FileName = RecogTextFile + ".txt"
        If ofd.ShowDialog() = Windows.Forms.DialogResult.OK Then
            RecogTextFile = IO.Path.GetFileNameWithoutExtension(ofd.FileName)
            ReadRecogFile()
        End If
    End Sub

    Friend Sub SpeechSave()
        Dim sfd As SaveFileDialog = New SaveFileDialog()
        sfd.InitialDirectory = SpeechPath
        sfd.DefaultExt = ".txt"
        sfd.AddExtension = True
        sfd.FileName = SpeechTextFile
        sfd.Filter = "Speech text file (*.txt)|*txt"
        If sfd.ShowDialog() = Windows.Forms.DialogResult.OK Then
            SpeechTextFile = IO.Path.GetFileNameWithoutExtension(sfd.FileName)
            SaveSpeechFile()
        End If
    End Sub
    Friend Sub SpeechLoad()
        Dim ofd As OpenFileDialog = New OpenFileDialog()
        ofd.InitialDirectory = SpeechPath
        ofd.Filter = "Speech text file (*.txt)|*txt"
        ofd.DefaultExt = ".txt"
        ofd.FileName = SpeechTextFile + ".txt"
        If ofd.ShowDialog() = Windows.Forms.DialogResult.OK Then
            SpeechTextFile = IO.Path.GetFileNameWithoutExtension(ofd.FileName)
            ReadSpeechFile()
        End If
    End Sub

    ' ==============================================================
    '  REPLACE MULTIPLE SPACES and TABS and TRIM
    ' ============================================================== 
    Friend Function ReplaceMultipleSpacesAndTrim(ByVal s As String) As String
        s = s.Replace(vbTab, " ")
        While s.Contains("  ")
            s = s.Replace("  ", " ")
        End While
        Return s.Trim
    End Function


    ' ==============================================================
    '  TextBox Helpers
    ' ==============================================================
    Friend Sub TextBox_SelectLine(ByVal tbox As TextBox, ByVal lineindex As Int32)
        Dim p1 As Int32 = tbox.GetFirstCharIndexFromLine(lineindex)
        If p1 < 0 Then Return
        Dim p2 As Int32 = tbox.Text.IndexOf(vbLf, p1)
        If p2 <= 0 Then p2 = tbox.Text.Length
        tbox.SelectionStart = p1
        tbox.SelectionLength = p2 - p1
        tbox.ScrollToCaret()
    End Sub

    Friend Sub TextBox_SelectWholeLine(ByVal tbox As TextBox)
        Dim p1 As Int32 = tbox.GetFirstCharIndexOfCurrentLine()
        If p1 < 0 Then Return
        Dim p2 As Int32 = tbox.Text.IndexOf(vbLf, p1)
        If p2 <= 0 Then p2 = tbox.Text.Length
        tbox.SelectionStart = p1
        tbox.SelectionLength = p2 - p1
    End Sub


End Module
